/****************************************************************************************/
/* Basis.ino                                                                            */
/* Collection of basic routines                                                         */
/* SUB_SEND_HOMEMATIC         Subroutine to send register value via homematic interface */
/* SUB_TRANS_HOMEMATIC        Subroutine to tranmit byte value via homematic interface  */
/****************************************************************************************/


/****************************************************************************************/
/* Sends the reister value of the CO2 value or airpressure value via the homematic      */
/* interface                                                                            */
/* D7  D6  D5  D4  D3  D2  D1  D0                                                       */
/*  0   0  x5  x4  x3  x2  x1  x0 --> LSB of CO2 value                                  */
/*                                    x0-x5 = Lower 6 bit of CO2 value                  */
/*  0   1  x11 x10 x9  x8  x7  x6 --> MSB of CO2 value                                  */
/*                                    x6-x11 = Higher 6 bit of CO2 value                */
/*  1   0  x5  x4  x3  x2  x1  x0 --> LSB of air pressure value                         */
/*                                    x0-x5 = Lower 6 bit of air pressure value         */
/*  1   1  x11 x10 x9  x8  x7  x6 --> MSB of air pressure value                         */
/*                                    x6-x11 = Higher 6 bit of air pressure value       */
/*                                                                                      */
/*      First parameter: 0 = CO2 value                                                  */
/*                       1 = Airpressure value                                          */
/****************************************************************************************/ 
void SUB_SEND_HOMEMATIC (byte TYPE_BYTE)
{
  unsigned int TEMP_VALUE;                        /* Termporary value for processing */
  byte         HM_VALUE;                          /* Value to be tranfered via the HM interface */

  /* Step 1: Transmit LSB */
  TEMP_VALUE = REGISTER_VALUE & 0x003F;           /* Mask 6 lowest bytes (LSB) */
  HM_VALUE = (byte)TEMP_VALUE;                    /* Convert LSB into byte */          
  if (TYPE_BYTE == 1)                             /* Airpressure value ? */ 
  {
    HM_VALUE = HM_VALUE | B10000000;              /* Set D7 bit 1 for airpressure LSB value */
    #ifdef _DEBUG_
      Serial.print ("  Sending airpressure value LSB:");
      Serial.println (HM_VALUE);
    # endif    
  }
  else                                            /* CO2 value ? */
  {
    #ifdef _DEBUG_
      Serial.print ("  Sending CO2 value LSB:");
      Serial.println (HM_VALUE);
    # endif     
  }
  SUB_TRANS_HOMEMATIC (HM_VALUE);                 /* Transmit value via Homematic interface */ 

  /* Step 2: Wait for one minute */
  delay (1000);                                   /* Wait one second for sending MSB */

  /* Step 3: Transmit MSB */
  TEMP_VALUE = REGISTER_VALUE & 0x0FC0;           /* Mask 6 highest bytes (MSB) */
  TEMP_VALUE = TEMP_VALUE >> 6;                   /* Shift 6 bytes left */
  HM_VALUE = (byte)TEMP_VALUE;                    /* Convert LSB into byte */         
  if (TYPE_BYTE == 1)                             /* Airpressure value ? */ 
  {
    HM_VALUE = HM_VALUE | B11000000;              /* Set D7 bit and D6 bit 1 for airpressure MSB value */
    #ifdef _DEBUG_
      Serial.print ("  Sending airpressure value MSB:");
      Serial.println (HM_VALUE);
    # endif 
  }
  else                                            /* CO2 value ? */
  {
    HM_VALUE = HM_VALUE | B01000000;              /* Set D6 bit 1 for CO2 MSB value */ 
    #ifdef _DEBUG_
      Serial.print ("  Sending CO2 value MSB:");
      Serial.println (HM_VALUE);
    # endif 
  }
  SUB_TRANS_HOMEMATIC (HM_VALUE);                 /* Transmit value via Homematic interface */    
}

 
/****************************************************************************************/
/* Transmits the byte via the homematic interface                                       */
/* interface                                                                            */
/*      First parameter: Byte value to be transmitted                                   */
/****************************************************************************************/ 
void SUB_TRANS_HOMEMATIC (byte TRANS_BYTE)
{
  /* Transmit D0 */
  if ((TRANS_BYTE & B00000001) == 0)
    digitalWrite(DATA_0, LOW);
  else
    digitalWrite(DATA_0, HIGH);
  /* Transmit D1 */
  if ((TRANS_BYTE & B00000010) == 0)
    digitalWrite(DATA_1, LOW);
  else
    digitalWrite(DATA_1, HIGH);
  /* Transmit D2 */
  if ((TRANS_BYTE & B00000100) == 0)
    digitalWrite(DATA_2, LOW);
  else
    digitalWrite(DATA_2, HIGH);  
  /* Transmit D3 */
  if ((TRANS_BYTE & B00001000) == 0)
    digitalWrite(DATA_3, LOW);
  else
    digitalWrite(DATA_3, HIGH); 
 /* Transmit D4 */
  if ((TRANS_BYTE & B00010000) == 0)
    digitalWrite(DATA_4, LOW);
  else
    digitalWrite(DATA_4, HIGH); 
 /* Transmit D5 */
  if ((TRANS_BYTE & B00100000) == 0)
    digitalWrite(DATA_5, LOW);
  else
    digitalWrite(DATA_5, HIGH); 
 /* Transmit D6 */
  if ((TRANS_BYTE & B01000000) == 0)
    digitalWrite(DATA_6, LOW);
  else
    digitalWrite(DATA_6, HIGH); 
 /* Transmit D7 */
  if ((TRANS_BYTE & B10000000) == 0)
    digitalWrite(DATA_7, LOW);
  else
    digitalWrite(DATA_7, HIGH);

  delay (10);                                     /* 10ms delay to stable data */
  digitalWrite(DATA_ACK, LOW);                    /* Set data acknowledge low */
  delay (300);                                    /* 300ms delay for acknowledge low delay */
  digitalWrite(DATA_ACK, HIGH);                   /* Set data acknowledge high */
}
