/****************************************************************************************/
/* Sensor.ino                                                                           */
/* Collection of Cubic CO2 and BM280 humidity sensor routines                           */
/* 1. CO2 communication routines:                                                       */
/* SUB_SENSOR_CO2_VALUE:      Subroutine to read CO2 value of Cubic CO2 sensor          */
/*                                                                                      */
/* 2. BME280 communication routines:                                                    */
/* SUB_SENSOR_BME280_VALUE    Subroutine to read BME280 air pressure                    */
/****************************************************************************************/


/****************************************************************************************/
/* Read CO2 value                                                                       */
/****************************************************************************************/
void SUB_SENSOR_CO2_VALUE ()
{  
  byte         READ_LOOP;                         /* Read loop variable */
  unsigned int TIMEOUT_COUNTER;                   /* Timeout counter */
  boolean      SENSOR_ERROR;                      /* Sensor error 0 = no error, 1 = error */
    
  #ifdef _DEBUG_
    Serial.println();
    Serial.println ("Reading CO2 value...");
    Serial.println ("  Wait for reception of data, this may take up to 30 Seconds");
  # endif

  SENSOR_ERROR = 0;                               /* Reset sensor error flag */
  REGISTER_VALUE = 0;                             /* Set register value to zero = Error */
  /* Empty receiver buffer */
  while (Serial1.available())                     /* Not empty receiver buffer? */
  {
    Serial1.read();                               /* Read buffer */
  }

  /* Step A: Wait for first reception of data or timeout of 40 seconds reached ? */
  TIMEOUT_COUNTER = 0;                            /* Reset timeout counter */
  do                                              /* Wait for first char */
  {
    delay(100);                                   /* 100ms delay */
    TIMEOUT_COUNTER++;                            /* Increase timeout counter by one */
  }
  while ((Serial1.available() == 0) && (TIMEOUT_COUNTER < 400));
  if (TIMEOUT_COUNTER >= 400)
  {
    #ifdef _DEBUG_
      Serial.println ("  Timeout!!! No data received within 40 seconds!");
    # endif 
    return;                                       /* Return from sub routine */
  }
  
  /* Step B: Wait for reception of all eight data bytes */
  TIMEOUT_COUNTER = 0;                            /* Reset timeout counter */
  /* Not all number of bytes received or timeout ? */
  while ((Serial1.available() < 8) && (SENSOR_ERROR == 0))                 
  {
    TIMEOUT_COUNTER++;
    if (TIMEOUT_COUNTER > 20)                     /* Timeout after 1 second ? */
    {
      SENSOR_ERROR = 1;                           /* Set sensor error flag true */
    }
    delay(50);                                    /* 50ms delay */
  }
  
  /* Step C: Reading loop */
  if (SENSOR_ERROR == 0)                          /* No error = Received all bytes ? */
  {
    for (READ_LOOP = 0; READ_LOOP < 8; READ_LOOP++)
    {
      SENSOR_RESPONSE[READ_LOOP] = Serial1.read();/* Store read byte in sensor response array */    
    }
    #ifdef _DEBUG_
      Serial.println ("  Read data from serial interface");
    # endif
  }
  else
  {
    #ifdef _DEBUG_
      Serial.println ("  Wrong number of data received!");
    # endif 
    return;                                       /* Return from subroutine */ 
  }  
  
  /* Step D: Check first parameter to be 0x16 */
  if (SENSOR_RESPONSE[0] != 0x16)
  {
    #ifdef _DEBUG_
      Serial.print ("  Wrong byte 1 read, should be 22 but is: ");
      Serial.println (SENSOR_RESPONSE[0]);
    # endif 
    return;                                       /* Return from subroutine */  
  }

  /* Step E: Check second parameter to be 0x05 */
  if (SENSOR_RESPONSE[1] != 0x05)
  {
    #ifdef _DEBUG_
      Serial.print ("  Wrong byte 2 read, should be 5 but is: ");
      Serial.println (SENSOR_RESPONSE[1]);
    # endif 
    return;                                       /* Return from subroutine */  
  }

  /* Step F: Check third parameter to be 0x01 */
  if (SENSOR_RESPONSE[2] != 0x01)
  {
    #ifdef _DEBUG_
      Serial.print ("  Wrong byte 3 read, should be 1 but is: ");
      Serial.println (SENSOR_RESPONSE[2]);
    # endif 
    return;                                       /* Return from subroutine */  
  }  

  /* Step G: Check checksum */
  REGISTER_VALUE = SENSOR_RESPONSE[0] + SENSOR_RESPONSE[1] + SENSOR_RESPONSE[2] + SENSOR_RESPONSE[3] + SENSOR_RESPONSE[4] + SENSOR_RESPONSE[5] + SENSOR_RESPONSE[6];
  REGISTER_VALUE = REGISTER_VALUE % 256;          /* Modulo operation */
  REGISTER_VALUE = 256 - REGISTER_VALUE;          /* Calculate CRC value */
  /* Check CRC */
  if (SENSOR_RESPONSE[7] != REGISTER_VALUE)
  {
    #ifdef _DEBUG_
      Serial.print ("  CRC error, should be ");
      Serial.print (SENSOR_RESPONSE[7]);
      Serial.print (" but is: ");
      Serial.println (REGISTER_VALUE);
    # endif 
    REGISTER_VALUE = 0;                           /* Set register value to zero = Error */
    return;                                       /* Return from subroutine */  
  } 

  /* Step H: Read CO2 value */
  REGISTER_VALUE = SENSOR_RESPONSE[3]*256 + SENSOR_RESPONSE[4];
  #ifdef _DEBUG_
    Serial.print ("  CO2 value: ");
    Serial.println (REGISTER_VALUE);
  # endif   
}


/****************************************************************************************/
/* Read BME280 sensor air pressure, temperature and humidity and display it             */
/****************************************************************************************/
void SUB_SENSOR_BME280_VALUE ()
{ 
  float READ_REGISTER_VALUE;                       /* Variable for air pressure value */

  /* Read sensor air pressure value and calculate air pressure value */
  READ_REGISTER_VALUE = bme.readPressure();
  READ_REGISTER_VALUE = bme.seaLevelForAltitude(ALTITUDE,READ_REGISTER_VALUE);
  READ_REGISTER_VALUE = READ_REGISTER_VALUE/100.0F;
  REGISTER_VALUE = round(READ_REGISTER_VALUE);    /* Convert value into int */
  #ifdef _DEBUG_
    Serial.print ("  Airpressure value:  ");
    Serial.print (READ_REGISTER_VALUE);
    Serial.print (" hPa, Rounded value: ");
    Serial.print (REGISTER_VALUE);
    Serial.println (" hPa");
  #endif
}
