/*************************************************************************************************************************************/
/* Interface for CO2 meter Technoline WL1030 with Homematic integration                                                              */
/* Version: 1.0                                                                                                                      */
/* Board: Ardino Leonardo                                                                                                            */
/* Date: 09.11.2021                                                                                                                  */
/* Compiled with Arduino 1.8.12                                                                                                      */
/*                                                                                                                                   */
/* Libraries:                                                                                                                        */
/*        Adafruit BME280 Library: https://github.com/adafruit/Adafruit_BME280_Library                                              `*/
/*        Adafruit Adafruit Sensor Library: https://github.com/adafruit/Adafruit_Sensor                                              */
/*                                                                                                                                   */
/* References:                                                                                                                       */
/*  Technoline WL1030:                                                                                                               */
/*  https://de.elv.com/technoline-co2-messgeraet-co2-anzeige-wl1030-kohlendioxid-mit-grafischer-ampel-funktion-251660?fs=2259309422  */
/*  Cubic Sensor sensor CM1106H-NS:                                                                                                  */
/*  https://en.gassensor.com.cn/CO2Sensor/info_itemid_88.html                                                                        */
/*  ELV homematic module: 8-Bit HM-MOD-EM-8Bit                                                                                       */
/*  https://de.elv.com/elv-homematic-komplettbausatz-funk-sendemodul-8-bit-hm-mod-em-8bit-                                           */
/*  fuer-smart-home-hausautomation-150253?fs=957109552&c=499                                                                         */
/*  BME280 module                                                                                                                    */
/*  https://www.makershop.de/sensoren/temperatur/bme280-5-volt/                                                                      */
/*                                                                                                                                   */
/*  Technoline WL1030 device sends data every 30 seconds                                                                             */
/*                                                                                                                                   */
/*  Communication format with Cubic                                                                                                  */
/*      Example reading sequence for CO2 response value: 0x16, 0X05, 0X01, DF1, DF2, DF3, DF4, Checksum                              */      
/*        Byte 01    = 0x16    Fixed to 0x16                                                                                         */
/*        Byte 02    = 0x05    Length of data (command+data) - Always 0x05                                                           */
/*        Byte 03    = 0x01    Command read measured result of CO2                                                                   */ 
/*        Byte 04    = CO2     CO2 high value                                                                                        */
/*        Byte 05    = CO2     CRC low value                                                                                         */
/*        Byte 06    = Status  Status bit                                                                                            */
/*        Byte 07    = Res.    Reserved                                                                                              */
/*        Byte 08    = Checksum                                                                                                      */
/*                                                                                                                                   */
/* Checksum = 256 - (Byte_01 + Byte_02 + Byte_03 + Byte_04 + Byte_05 + Byte_06 + Byte_07)Modulo(%) 256                               */
/*                                                                                                                                   */
/* Parameter of serial interfaces for CO2 sensor:                                                                                    */
/*        9600 Baud, 8 data bits, no parity, 1 stop bit, no flow control                                                             */
/*        Serial: ON board converter --> 115.000 buad                                                                                */
/*        Serial 1: Hardware interface to connect sensor                                                                             */
/*                                                                                                                                   */
/* Pin asignments:                                                                                                                   */
/*     RxI  = Serial 1 RXD --> Connect to Cubic sendor TxD pin                                                                       */
/*     IO2  = SDA                                                                                                                    */
/*     IO3  = SCL                                                                                                                    */
/*     IO4  = Homematic D0                                                                                                           */
/*     IO5  = Homematic D1                                                                                                           */                        
/*     IO6  = Homematic D2                                                                                                           */
/*     IO7  = Homematic D3                                                                                                           */
/*     IO8  = Homematic D4                                                                                                           */
/*     IO9  = Homematic D5                                                                                                           */
/*     IO10 = Homematic D6                                                                                                           */
/*     IO14 = Homematic D7                                                                                                           */
/*     IO16 = Homematic DATA_ACK                                                                                                     */
/*                                                                                                                                   */
/*************************************************************************************************************************************/


/****************************************************************************************/
/* Libraries                                                                            */
/****************************************************************************************/
/* Include libraries for BME280 sensor */ 
#include <Adafruit_Sensor.h>                      /* Include Adafruit sensor library --> Must be in Arduino Library directory */
#include "src/BME280/src/Adafruit_BME280.h"       /* Include BME280 sensor library */

/****************************************************************************************/
/* Defintions and initialisation for BM280 sensor                                       */
/****************************************************************************************/
#define SEALEVELPRESSURE_HPA (1013.25)            /* Definition of sealevel pressure */
#define BME280_ADD 0x76                           /* Define I2C address */
#define ALTITUDE 180.0                            /* Definition of altitude in Vienna Austria */
Adafruit_BME280 bme;                              /* Definition for BME280 sensor */        

/****************************************************************************************/
/* Definitions of hardware pins                                                         */
/****************************************************************************************/
#define DATA_0           4                        /* Homematic D0 pin */
#define DATA_1           5                        /* Homematic D1 pin */
#define DATA_2           6                        /* Homematic D2 pin */
#define DATA_3           7                        /* Homematic D3 pin */
#define DATA_4           8                        /* Homematic D4 pin */
#define DATA_5           9                        /* Homematic D5 pin */
#define DATA_6          10                        /* Homematic D6 pin */
#define DATA_7          14                        /* Homematic D7 pin */
#define DATA_ACK        16                        /* Homematic DU3 data acknowledge pin */

/****************************************************************************************/
/* Definitions for general settings                                                     */
/****************************************************************************************/
/* If this is uncommented here, status messages will be output via the serial interface, if commented out then disabled ! */
#define _DEBUG_                                   

/****************************************************************************************/
/* Constants                                                                            */
/****************************************************************************************/
const char* SW_VERSION = "1.0";                   /* Actual SW version */
const byte  TRANSMIT_COUNT = 10;                  /* Wait 10*30sec = 5 min receipes until transmission via homematic interface */

/****************************************************************************************/
/* Variables                                                                            */
/****************************************************************************************/
unsigned int REGISTER_VALUE;                      /* Actual register value of CO2 and air pressure */
boolean      BME280_PRESENT;                      /* True = BME280 present, False = BME280 not present */
byte         SENSOR_RESPONSE[10];                 /* Reserve buffer of 10 byte for answer from Cubic CO2 sensor */
byte         TRANSMIT_COUNTER;                    /* Check number of transmissions */


/*******************************************************************************************/
/*                                                                                         */
/* Setup routine                                                                           */
/*                                                                                         */
/*******************************************************************************************/
void setup() 
{       
  Serial.begin(115200);                           /* Start serial interface for terminal communication at 115000 Baud */
  delay (10000);                                  /* Wait for 10 seconds */
  /* Display welcome message with initialisation information */
  Serial.println("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n");
  Serial.println ("*********************************************************");
  Serial.println ("* CO2 Meter with Cubic sensor                           *");
  Serial.println ("*                                                       *");
  Serial.print   ("* V");
  Serial.print   (SW_VERSION);
  Serial.println ("                                      @PT 2021    *");   
  Serial.println ("*********************************************************\n"); 

  #ifdef _DEBUG_
    Serial.println("\n\n\n"); 
    Serial.println ("Debug is enabled...");
    Serial.println(); 
  #else
    Serial.println("\n\n\n"); 
    Serial.println ("Debug is disabled...");
    Serial.println ("Remove commented out line #define _DEBUG_ to enable!");
    Serial.println();     
  # endif  

  /* Step 1: Prepare homematic interface */
  #ifdef _DEBUG_
    Serial.println("  Step 1: Initializing homematic interface: Done");
  # endif   
  digitalWrite(DATA_ACK, HIGH);                   /* Set data acknowlwdge to high */
  pinMode(DATA_ACK, OUTPUT);                      /* Define Data Acknowledge as output */
  digitalWrite(DATA_ACK, HIGH);                   /* Set data acknowlwdge to high */
  pinMode(DATA_0, OUTPUT);                        /* Define D0 as output */
  digitalWrite(DATA_0, HIGH);                     /* Set D0 to high */
  pinMode(DATA_1, OUTPUT);                        /* Define D1 as output */
  digitalWrite(DATA_1, HIGH);                     /* Set D1 to high */
  pinMode(DATA_2, OUTPUT);                        /* Define D2 as output */
  digitalWrite(DATA_2, HIGH);                     /* Set D2 to high */
  pinMode(DATA_3, OUTPUT);                        /* Define D3 as output */
  digitalWrite(DATA_3, HIGH);                     /* Set D3 to high */
  pinMode(DATA_4, OUTPUT);                        /* Define D4 as output */
  digitalWrite(DATA_4, HIGH);                     /* Set D4 to high */
  pinMode(DATA_5, OUTPUT);                        /* Define D5 as output */
  digitalWrite(DATA_5, HIGH);                     /* Set D5 to high */
  pinMode(DATA_6, OUTPUT);                        /* Define D6 as output */
  digitalWrite(DATA_6, HIGH);                     /* Set D6 to high */
  pinMode(DATA_7, OUTPUT);                        /* Define D7 as output */
  digitalWrite(DATA_7, HIGH);                     /* Set D7 to high */   

  /* Step 2: Initialize Cubic CO2 sensor serial interface */
  #ifdef _DEBUG_
    Serial.println ("  Step 2: Init Cubic CO2 interface: Done");
  # endif
  Serial1.begin(9600, SERIAL_8N1);                /* Intitialize serial interface for Cubic CO2 sensor */

  /* Step 3: Initialize BME280 pressure sensor */
  #ifdef _DEBUG_
    Serial.print("  Step 3: Initializing BME280 air pressure sensor: ");
  # endif 
  BME280_PRESENT = bme.begin(BME280_ADD);         /* Init air pressure sensor */
  if (BME280_PRESENT == false)                    /* Sensor not found ? */
  {
    #ifdef _DEBUG_
      Serial.println("Failed");
    # endif
  }
  else                                            /* Sensor found ? */
  {
    #ifdef _DEBUG_
      Serial.println("Done");
    # endif
  } 

  /* Initialisation finished */
  #ifdef _DEBUG_
    Serial.println();
    Serial.println ("Initialisation finished!!!");
    Serial.println();
  # endif  
}


/*******************************************************************************************/
/*                                                                                         */
/* Main loop                                                                               */
/*                                                                                         */
/*******************************************************************************************/
void loop()
{
  TRANSMIT_COUNTER = 0;                           /* Set transmission counter to 0 */
  do                                              /* Endless loop */
  {
    SUB_SENSOR_CO2_VALUE ();                      /* Call subroutine to read CO2 value */
    TRANSMIT_COUNTER++;                           /* Increase transmit counter by 1 */
    if (TRANSMIT_COUNTER == TRANSMIT_COUNT)       /* Transmit count reached? --> Transmit via Homematic interface */
    {
      #ifdef _DEBUG_
        Serial.println ("Transmitting CO2 value via homematic interface ...");
      # endif       
      SUB_SEND_HOMEMATIC (0);                     /* Call subroutine to transmit CO2 value via Homematic interface */
    }
    if (BME280_PRESENT == true)                   /* BME280 sensor present ? */
    {   
      delay (1000);                               /* Wait one second for receiving airpressure value */
      SUB_SENSOR_BME280_VALUE ();                 /* Call subroutine to read airpressure */
      if (TRANSMIT_COUNTER == TRANSMIT_COUNT)     /* Transmit count reached? --> Transmit via Homematic interface */
      {
        #ifdef _DEBUG_
          Serial.println ("Transmitting airpressure value via homematic interface ...");
        # endif       
        SUB_SEND_HOMEMATIC (1);                   /* Call subroutine to transmit airpressure value via Homematic interface */
      }
    }
    if (TRANSMIT_COUNTER == TRANSMIT_COUNT)       /* Transmit count reached? --> Transmit via Homematic interface */
    {
      TRANSMIT_COUNTER = 0;                       /* Set transmission counter to 0 */
    }
  }
  while (1);  
}
